/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
package com.examples.anyuri;

import com.examples.anyuri.po.*;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.InputStream;
import java.net.URI;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class PurchaseOrderService extends IPurchaseOrder_Impl {

    public PurchaseOrderStatus acceptPO(URI uri) throws POProcessingProblem, RemoteException {
        if (uri == null) {
            throw new POProcessingProblem("Web Service was passed a null URI for a Purchase order");
        }
        System.out.println("PurchaseOrder received in the web service" + uri);
        try {
            InputStream in = uri.toURL().openStream();
            PurchaseOrder order = toObject(in);
            System.out.println("JAXB Unmarshalled the purchase Order ");
            System.out.println("PurchaseOrder received in the web service");
            System.out.println("Order ID " + order.getPoID());
            System.out.println("Order creation date " + order.getCreateDate());
            System.out.println("Ship To Address " + order.getShipTo());
            System.out.println("Bill To Address " + order.getBillTo());
            System.out.println("Order Contents :");
            List items = order.getItems();
            Iterator it = items.iterator();
            while (it.hasNext()) {
                LineItem item = (LineItem) it.next();
                System.out.println("Item Name =" + item.getItemname() + " Price=" + item.getPrice() + " Quantity=" + item.getQuantity());
            }
        } catch (Exception ex) {
            throw new POProcessingProblem("Purchase order cannot be processed " + ex);
        }

        PurchaseOrderStatus status = new PurchaseOrderStatus();
        status.setTimestamp(new Date().toString());
        status.setOrderid("ABC" + System.currentTimeMillis());
        return status;
    }


    /**
     * Unmarshall the stream to its JAXB object representation
     *
     * @param n
     * @throws Exception
     */
    public PurchaseOrder toObject(InputStream in) throws Exception {
        JAXBContext jc = JAXBContext.newInstance("com.examples.anyuri.po");
        Unmarshaller u = jc.createUnmarshaller();
        // enable validation
        u.setValidating(true);
        Object o = u.unmarshal(in);
        // use the object that is generated by JAXB
        PurchaseOrder order = (PurchaseOrder) o;
        return order;
    }


}
